@@ -14,15 +14,15 @@ from pyshacl.pytypes import GraphLike
 from pyshacl.rdfutil import stringify_node
 
 
-SH_not = SH.term('not')
-SH_and = SH.term('and')
-SH_or = SH.term('or')
-SH_xone = SH.term('xone')
+SH_not = SH["not"]
+SH_and = SH["and"]
+SH_or = SH["or"]
+SH_xone = SH.xone
 
-SH_NotConstraintComponent = SH.term('NotConstraintComponent')
-SH_AndConstraintComponent = SH.term('AndConstraintComponent')
-SH_OrConstraintComponent = SH.term('OrConstraintComponent')
-SH_XoneConstraintComponent = SH.term('XoneConstraintComponent')
+SH_NotConstraintComponent = SH.NotConstraintComponent
+SH_AndConstraintComponent = SH.AndConstraintComponent
+SH_OrConstraintComponent = SH.OrConstraintComponent
+SH_XoneConstraintComponent = SH.XoneConstraintComponent
 
 
 class NotConstraintComponent(ConstraintComponent):
@@ -34,6 +34,10 @@ class NotConstraintComponent(ConstraintComponent):
     For each value node v: A failure MUST be reported if the conformance checking of v against the shape $not produces a failure. Otherwise, if v conforms to the shape $not, there is validation result with v as sh:value.
     """
 
+    shacl_constraint_component = SH_NotConstraintComponent
+    shape_expecting = True
+    list_taking = False
+
     def __init__(self, shape):
         super(NotConstraintComponent, self).__init__(shape)
         not_list = list(self.shape.objects(SH_not))
@@ -57,61 +61,62 @@ class NotConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "NotConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_NotConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         m = "Node {} conforms to shape {}".format(
             stringify_node(datagraph, value_node), stringify_node(self.shape.sg.graph, self.not_list[0])
         )
         return [rdflib.Literal(m)]
 
-    def evaluate(self, target_graph: GraphLike, focus_value_nodes: Dict, _evaluation_path: List):
+    def evaluate(self, datagraph: GraphLike, focus_value_nodes: Dict, _evaluation_path: List):
         """
 
         :type focus_value_nodes: dict
-        :type target_graph: rdflib.Graph
-        :type _evaluation_path list
+        :type datagraph: rdflib.Graph
+        :type _evaluation_path: List
         """
         reports = []
         non_conformant = False
-        shape = self.shape
         potentially_recursive = self.recursion_triggers(_evaluation_path)
 
-        def _evaluate_not_constraint(not_c):
-            nonlocal self, shape, target_graph, focus_value_nodes, _evaluation_path
-            _reports = []
-            _non_conformant = False
-            not_shape = shape.get_other_shape(not_c)
-            if not not_shape:
-                raise ReportableRuntimeError(
-                    "Shape pointed to by sh:not does not exist or is not " "a well-formed SHACL Shape."
-                )
-            if not_shape in potentially_recursive:
-                warn(ShapeRecursionWarning(_evaluation_path))
-                return _non_conformant, _reports
-            for f, value_nodes in focus_value_nodes.items():
-                for v in value_nodes:
-                    try:
-                        _is_conform, _r = not_shape.validate(
-                            target_graph, focus=v, _evaluation_path=_evaluation_path[:]
-                        )
-                    except ValidationFailure as e:
-                        raise e
-                    if _is_conform:
-                        # in this case, we _dont_ want to conform!
-                        _non_conformant = True
-                        rept = self.make_v_result(target_graph, f, value_node=v)
-                        _reports.append(rept)
-            return _non_conformant, _reports
-
         for not_c in self.not_list:
-            _nc, _r = _evaluate_not_constraint(not_c)
+            _nc, _r = self._evaluate_not_constraint(
+                not_c, datagraph, focus_value_nodes, potentially_recursive, _evaluation_path
+            )
             non_conformant = non_conformant or _nc
             reports.extend(_r)
         return (not non_conformant), reports
 
+    def _evaluate_not_constraint(self, not_c, datagraph, focus_value_nodes, potentially_recursive, _evaluation_path):
+        """
+        :type not_c: List[Node]
+        :type datagraph: rdflib.Graph
+        :type focus_value_nodes: dict
+        :type potentially_recursive: Optional[List]
+        :type _evaluation_path: List
+        """
+        _reports = []
+        _non_conformant = False
+        not_shape = self.shape.get_other_shape(not_c)
+        if not not_shape:
+            raise ReportableRuntimeError(
+                "Shape pointed to by sh:not does not exist or is not a well-formed SHACL Shape."
+            )
+        if potentially_recursive and not_shape in potentially_recursive:
+            warn(ShapeRecursionWarning(_evaluation_path))
+            return _non_conformant, _reports
+        for f, value_nodes in focus_value_nodes.items():
+            for v in value_nodes:
+                try:
+                    _is_conform, _r = not_shape.validate(datagraph, focus=v, _evaluation_path=_evaluation_path[:])
+                except ValidationFailure as e:
+                    raise e
+                if _is_conform:
+                    # in this case, we _dont_ want to conform!
+                    _non_conformant = True
+                    rept = self.make_v_result(datagraph, f, value_node=v)
+                    _reports.append(rept)
+        return _non_conformant, _reports
+
 
 class AndConstraintComponent(ConstraintComponent):
     """
@@ -122,6 +127,10 @@ class AndConstraintComponent(ConstraintComponent):
     For each value node v: A failure MUST be produced if the conformance checking of v against any of the members of $and produces a failure. Otherwise, if v does not conform to each member of $and, there is a validation result with v as sh:value.
     """
 
+    shacl_constraint_component = SH_AndConstraintComponent
+    shape_expecting = True
+    list_taking = True
+
     def __init__(self, shape):
         super(AndConstraintComponent, self).__init__(shape)
         and_list = list(self.shape.objects(SH_and))
@@ -140,10 +149,6 @@ class AndConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "AndConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_AndConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         and_list = " , ".join(
             stringify_node(self.shape.sg.graph, a_c) for a in self.and_list for a_c in self.shape.sg.graph.items(a)
@@ -169,13 +174,13 @@ class AndConstraintComponent(ConstraintComponent):
             sg = shape.sg.graph
             and_list = set(sg.items(and_c))
             if len(and_list) < 1:
-                raise ReportableRuntimeError("The list associated with sh:and is not a " "valid RDF list.")
+                raise ReportableRuntimeError("The list associated with sh:and is not a valid RDF list.")
             and_shapes = set()
             for a in and_list:
                 and_shape = shape.get_other_shape(a)
                 if not and_shape:
                     raise ReportableRuntimeError(
-                        "Shape pointed to by sh:and does not exist or " "is not a well-formed SHACL Shape."
+                        "Shape pointed to by sh:and does not exist or is not a well-formed SHACL Shape."
                     )
                 and_shapes.add(and_shape)
             for f, value_nodes in focus_value_nodes.items():
@@ -211,6 +216,10 @@ class OrConstraintComponent(ConstraintComponent):
     For each value node v: A failure MUST be produced if the conformance checking of v against any of the members produces a failure. Otherwise, if v conforms to none of the members of $or there is a validation result with v as sh:value.
     """
 
+    shacl_constraint_component = SH_OrConstraintComponent
+    shape_expecting = True
+    list_taking = True
+
     def __init__(self, shape):
         super(OrConstraintComponent, self).__init__(shape)
         or_list = list(self.shape.objects(SH_or))
@@ -229,10 +238,6 @@ class OrConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "OrConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_OrConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         or_list = " , ".join(
             stringify_node(self.shape.sg.graph, o_c) for o in self.or_list for o_c in self.shape.sg.graph.items(o)
@@ -259,13 +264,13 @@ class OrConstraintComponent(ConstraintComponent):
             sg = shape.sg.graph
             or_list = set(sg.items(or_c))
             if len(or_list) < 1:
-                raise ReportableRuntimeError("The list associated with sh:or " "is not a valid RDF list.")
+                raise ReportableRuntimeError("The list associated with sh:or is not a valid RDF list.")
             or_shapes = set()
             for o in or_list:
                 or_shape = shape.get_other_shape(o)
                 if not or_shape:
                     raise ReportableRuntimeError(
-                        "Shape pointed to by sh:or does not exist or " "is not a well-formed SHACL Shape."
+                        "Shape pointed to by sh:or does not exist or is not a well-formed SHACL Shape."
                     )
                 or_shapes.add(or_shape)
             for f, value_nodes in focus_value_nodes.items():
@@ -301,6 +306,10 @@ class XoneConstraintComponent(ConstraintComponent):
     For each value node v: A failure MUST be produced if the conformance checking of v against any of the members produces a failure. Otherwise, if v conforms to none of the members of $or there is a validation result with v as sh:value.
     """
 
+    shacl_constraint_component = SH_XoneConstraintComponent
+    shape_expecting = True
+    list_taking = True
+
     def __init__(self, shape):
         super(XoneConstraintComponent, self).__init__(shape)
         xone_nodes = list(self.shape.objects(SH_xone))
@@ -319,10 +328,6 @@ class XoneConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "XoneConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_XoneConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         xone_list = " , ".join(
             stringify_node(self.shape.sg.graph, a_c) for a in self.xone_nodes for a_c in self.shape.sg.graph.items(a)
@@ -337,7 +342,7 @@ class XoneConstraintComponent(ConstraintComponent):
 
         :type focus_value_nodes: dict
         :type target_graph: rdflib.Graph
-        :type _evaluation_path list
+        :type _evaluation_path: List
         """
         reports = []
         non_conformant = False
@@ -350,13 +355,13 @@ class XoneConstraintComponent(ConstraintComponent):
             sg = shape.sg.graph
             xone_list = list(sg.items(xone_c))
             if len(xone_list) < 1:
-                raise ReportableRuntimeError("The list associated with sh:xone is not " "a valid RDF list.")
+                raise ReportableRuntimeError("The list associated with sh:xone is not a valid RDF list.")
             xone_shapes = list()
             for x in xone_list:
                 xone_shape = shape.get_other_shape(x)
                 if not xone_shape:
                     raise ReportableRuntimeError(
-                        "Shape pointed to by sh:xone does not exist " "or is not a well-formed SHACL Shape."
+                        "Shape pointed to by sh:xone does not exist or is not a well-formed SHACL Shape."
                     )
                 xone_shapes.append(xone_shape)
             for f, value_nodes in focus_value_nodes.items():
