@@ -2,26 +2,24 @@
 """
 https://www.w3.org/TR/shacl/#core-components-others
 """
-from typing import Dict, List
+from typing import Dict, List, cast
 
 import rdflib
 
-from rdflib.namespace import RDFS
-
 from pyshacl.constraints.constraint_component import ConstraintComponent
-from pyshacl.consts import SH, RDF_type, SH_property
+from pyshacl.consts import RDFS, SH, RDF_type, SH_property
 from pyshacl.errors import ConstraintLoadError, ReportableRuntimeError
-from pyshacl.pytypes import GraphLike
+from pyshacl.pytypes import GraphLike, RDFNode
 from pyshacl.rdfutil import stringify_node
 
 
-SH_InConstraintComponent = SH.term('InConstraintComponent')
-SH_ClosedConstraintComponent = SH.term('ClosedConstraintComponent')
-SH_HasValueConstraintComponent = SH.term('HasValueConstraintComponent')
-SH_in = SH.term('in')
-SH_closed = SH.term('closed')
-SH_ignoredProperties = SH.term('ignoredProperties')
-SH_hasValue = SH.term('hasValue')
+SH_InConstraintComponent = SH.InConstraintComponent
+SH_ClosedConstraintComponent = SH.ClosedConstraintComponent
+SH_HasValueConstraintComponent = SH.HasValueConstraintComponent
+SH_in = SH["in"]
+SH_closed = SH.closed
+SH_ignoredProperties = SH.ignoredProperties
+SH_hasValue = SH.hasValue
 
 
 class InConstraintComponent(ConstraintComponent):
@@ -33,6 +31,10 @@ class InConstraintComponent(ConstraintComponent):
     For each value node that is not a member of $in, there is a validation result with the value node as sh:value.
     """
 
+    shacl_constraint_component = SH_InConstraintComponent
+    shape_expecting = False
+    list_taking = True
+
     def __init__(self, shape):
         super(InConstraintComponent, self).__init__(shape)
         in_vals = list(self.shape.objects(SH_in))
@@ -60,10 +62,6 @@ class InConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "InConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_InConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         list1 = [stringify_node(self.shape.sg.graph, val) for val in self.in_vals]
         m = "Value {} not in list {}".format(stringify_node(datagraph, value_node), list1)
@@ -96,7 +94,9 @@ class ClosedConstraintComponent(ConstraintComponent):
     If $closed is true then there is a validation result for each triple that has a value node as its subject and a predicate that is not explicitly enumerated as a value of sh:path in any of the property shapes declared via sh:property at the current shape. If $ignoredProperties has a value then the properties enumerated as members of this SHACL list are also permitted for the value node. The validation result MUST have the predicate of the triple as its sh:resultPath, and the object of the triple as its sh:value.
     """
 
-    ALWAYS_IGNORE = {(RDF_type, RDFS.term('Resource'))}
+    shacl_constraint_component = SH_ClosedConstraintComponent
+
+    ALWAYS_IGNORE = {(RDF_type, RDFS.Resource)}
 
     def __init__(self, shape):
         super(ClosedConstraintComponent, self).__init__(shape)
@@ -138,10 +138,6 @@ class ClosedConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "ClosedConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_ClosedConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         m = "Node {} is closed. It cannot have value: {}".format(
             stringify_node(datagraph, focus_node), stringify_node(datagraph, value_node)
@@ -184,7 +180,9 @@ class ClosedConstraintComponent(ConstraintComponent):
                     elif p in working_paths:
                         continue
                     non_conformant = True
-                    rept = self.make_v_result(target_graph, f, value_node=o, result_path=p)
+                    o_node = cast(RDFNode, o)
+                    p_node = cast(RDFNode, p)
+                    rept = self.make_v_result(target_graph, f, value_node=o_node, result_path=p_node)
                     reports.append(rept)
         return (not non_conformant), reports
 
@@ -198,6 +196,8 @@ class HasValueConstraintComponent(ConstraintComponent):
     If the RDF term $hasValue is not among the value nodes, there is a validation result.
     """
 
+    shacl_constraint_component = SH_HasValueConstraintComponent
+
     def __init__(self, shape):
         super(HasValueConstraintComponent, self).__init__(shape)
         has_value_set = set(self.shape.objects(SH_hasValue))
@@ -216,10 +216,6 @@ class HasValueConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "HasValueConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_HasValueConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         the_set = [stringify_node(self.shape.sg.graph, s) for s in self.has_value_set]
         p = self.shape.path()
