@@ -8,7 +8,7 @@ from warnings import warn
 import rdflib
 
 from pyshacl.constraints.constraint_component import ConstraintComponent
-from pyshacl.consts import SH, SH_node, SH_property
+from pyshacl.consts import SH, SH_node, SH_NodeConstraintComponent, SH_property, SH_PropertyConstraintComponent
 from pyshacl.errors import (
     ConstraintLoadError,
     ConstraintLoadWarning,
@@ -20,17 +20,14 @@ from pyshacl.pytypes import GraphLike
 from pyshacl.rdfutil import stringify_node
 
 
-SH_PropertyConstraintComponent = SH.term('PropertyConstraintComponent')
-SH_NodeConstraintComponent = SH.term('NodeConstraintComponent')
+SH_QualifiedValueCountConstraintComponent = SH.QualifiedValueConstraintComponent
+SH_QualifiedMaxCountConstraintComponent = SH.QualifiedMaxCountConstraintComponent
+SH_QualifiedMinCountConstraintComponent = SH.QualifiedMinCountConstraintComponent
 
-SH_QualifiedValueCountConstraintComponent = SH.term('QualifiedValueConstraintComponent')
-SH_QualifiedMaxCountConstraintComponent = SH.term('QualifiedMaxCountConstraintComponent')
-SH_QualifiedMinCountConstraintComponent = SH.term('QualifiedMinCountConstraintComponent')
-
-SH_qualifiedValueShape = SH.term('qualifiedValueShape')
-SH_qualifiedValueShapesDisjoint = SH.term('qualifiedValueShapesDisjoint')
-SH_qualifiedMinCount = SH.term('qualifiedMinCount')
-SH_qualifiedMaxCount = SH.term('qualifiedMaxCount')
+SH_qualifiedValueShape = SH.qualifiedValueShape
+SH_qualifiedValueShapesDisjoint = SH.qualifiedValueShapesDisjoint
+SH_qualifiedMinCount = SH.qualifiedMinCount
+SH_qualifiedMaxCount = SH.qualifiedMaxCount
 
 
 class PropertyConstraintComponent(ConstraintComponent):
@@ -42,6 +39,10 @@ class PropertyConstraintComponent(ConstraintComponent):
     For each value node v: A failure MUST be produced if the validation of v as focus node against the property shape $property produces a failure. Otherwise, the validation results are the results of validating v as focus node against the property shape $property.
     """
 
+    shacl_constraint_component = SH_PropertyConstraintComponent
+    shape_expecting = True
+    list_taking = False
+
     def __init__(self, shape):
         super(PropertyConstraintComponent, self).__init__(shape)
         property_shapes = list(self.shape.objects(SH_property))
@@ -60,10 +61,6 @@ class PropertyConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "PropertyConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_PropertyConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         raise NotImplementedError("A Property Constraint Component should not be able to generate its own message.")
 
@@ -74,22 +71,30 @@ class PropertyConstraintComponent(ConstraintComponent):
         :type target_graph: rdflib.Graph
         :type _evaluation_path: list
         """
-        reports = []
+        reports: List[Dict] = []
         non_conformant = False
         shape = self.shape
+
+        # Shortcut, when there are no value nodes, don't check for recursion, don't validate and exit early
+        value_node_count = 0
+        for f, value_nodes in focus_value_nodes.items():
+            value_node_count = value_node_count + len(value_nodes)
+        if value_node_count < 1:
+            return (not non_conformant), reports
+
         potentially_recursive = self.recursion_triggers(_evaluation_path)
 
         def _evaluate_property_shape(prop_shape):
-            nonlocal shape, target_graph, focus_value_nodes, _evaluation_path
+            nonlocal shape, target_graph, focus_value_nodes, _evaluation_path, potentially_recursive
             _reports = []
             _non_conformant = False
             prop_shape = shape.get_other_shape(prop_shape)
-            if prop_shape in potentially_recursive:
+            if potentially_recursive and prop_shape in potentially_recursive:
                 warn(ShapeRecursionWarning(_evaluation_path))
                 return _non_conformant, _reports
             if not prop_shape or not prop_shape.is_property_shape:
                 raise ReportableRuntimeError(
-                    "Shape pointed to by sh:property does not exist " "or is not a well-formed SHACL PropertyShape."
+                    "Shape pointed to by sh:property does not exist or is not a well-formed SHACL PropertyShape."
                 )
 
             for f, value_nodes in focus_value_nodes.items():
@@ -115,6 +120,10 @@ class NodeConstraintComponent(ConstraintComponent):
     For each value node v: A failure MUST be produced if the conformance checking of v against $node produces a failure. Otherwise, if v does not conform to $node, there is a validation result with v as sh:value.
     """
 
+    shacl_constraint_component = SH_NodeConstraintComponent
+    shape_expecting = True
+    list_taking = False
+
     def __init__(self, shape):
         super(NodeConstraintComponent, self).__init__(shape)
         node_shapes = list(self.shape.objects(SH_node))
@@ -133,15 +142,11 @@ class NodeConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "NodeConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_NodeConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         if len(self.node_shapes) < 2:
             m = "Value does not conform to Shape {}".format(stringify_node(self.shape.sg.graph, self.node_shapes[0]))
         else:
-            rules = "', '".join(stringify_node(self.shape.sg.graph, self.node_shapes[0]) for c in self.node_shapes)
+            rules = "', '".join(stringify_node(self.shape.sg.graph, c) for c in self.node_shapes)
             m = "Value does not conform to every Shape in ('{}')".format(rules)
         return [rdflib.Literal(m)]
 
@@ -152,22 +157,30 @@ class NodeConstraintComponent(ConstraintComponent):
         :type target_graph: rdflib.Graph
         :type _evaluation_path: list
         """
-        reports = []
+        reports: List[Dict] = []
         non_conformant = False
         shape = self.shape
+
+        # Shortcut, when there are no value nodes, don't check for recursion, don't validate and exit early
+        value_node_count = 0
+        for f, value_nodes in focus_value_nodes.items():
+            value_node_count = value_node_count + len(value_nodes)
+        if value_node_count < 1:
+            return (not non_conformant), reports
+
         potentially_recursive = self.recursion_triggers(_evaluation_path)
 
         def _evaluate_node_shape(node_shape):
-            nonlocal self, target_graph, shape, focus_value_nodes, _evaluation_path
+            nonlocal self, target_graph, shape, focus_value_nodes, _evaluation_path, potentially_recursive
             _reports = []
             _non_conformant = False
             node_shape = shape.get_other_shape(node_shape)
-            if node_shape in potentially_recursive:
+            if potentially_recursive and node_shape in potentially_recursive:
                 warn(ShapeRecursionWarning(_evaluation_path))
                 return _non_conformant, _reports
             if not node_shape or node_shape.is_property_shape:
                 raise ReportableRuntimeError(
-                    "Shape pointed to by sh:node does not exist or " "is not a well-formed SHACL NodeShape."
+                    "Shape pointed to by sh:node does not exist or is not a well-formed SHACL NodeShape."
                 )
             for f, value_nodes in focus_value_nodes.items():
                 for v in value_nodes:
@@ -197,6 +210,10 @@ class QualifiedValueShapeConstraintComponent(ConstraintComponent):
     Let C be as defined for sh:qualifiedMinCount above. A failure MUST be produced if any of the said conformance checks produces a failure. Otherwise, there is a validation result if C is greater than $qualifiedMaxCount. The constraint component for sh:qualifiedMaxCount is sh:QualifiedMaxCountConstraintComponent.
     """
 
+    shacl_constraint_component = NotImplemented
+    shape_expecting = True
+    list_taking = False
+
     def __init__(self, shape):
         super(QualifiedValueShapeConstraintComponent, self).__init__(shape)
         if not shape.is_property_shape:
@@ -261,14 +278,6 @@ class QualifiedValueShapeConstraintComponent(ConstraintComponent):
     def constraint_name(cls):
         return "QualifiedValueShapeConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        raise NotImplementedError(
-            "QualifiedValueShapeConstraintComponent must be either "
-            "QualifiedMinCountConstraintComponent or "
-            "QualifiedMaxCountConstraintComponent"
-        )
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         # TODO:
         #  Implement default message for QualifiedValueConstraint (seems messy)
@@ -279,24 +288,32 @@ class QualifiedValueShapeConstraintComponent(ConstraintComponent):
 
         :type focus_value_nodes: dict
         :type target_graph: rdflib.Graph
-        :type _evaluation_path list
+        :type _evaluation_path: List
         """
-        reports = []
+        reports: List[Dict] = []
         non_conformant = False
         shape = self.shape
+
+        # Shortcut, when there are no value nodes, don't check for recursion, don't validate and exit early
+        value_node_count = 0
+        for f, value_nodes in focus_value_nodes.items():
+            value_node_count = value_node_count + len(value_nodes)
+        if value_node_count < 1:
+            return (not non_conformant), reports
+
         potentially_recursive = self.recursion_triggers(_evaluation_path)
 
         def _evaluate_value_shape(_v_shape):
-            nonlocal self, shape, target_graph, focus_value_nodes, _evaluation_path
+            nonlocal self, shape, target_graph, focus_value_nodes, _evaluation_path, potentially_recursive
             _reports = []
             _non_conformant = False
             other_shape = shape.get_other_shape(_v_shape)
-            if other_shape in potentially_recursive:
+            if potentially_recursive and other_shape in potentially_recursive:
                 warn(ShapeRecursionWarning(_evaluation_path))
                 return _non_conformant, _reports
             if not other_shape:
                 raise ReportableRuntimeError(
-                    "Shape pointed to by sh:property does not " "exist or is not a well-formed SHACL Shape."
+                    "Shape pointed to by sh:property does not exist or is not a well-formed SHACL Shape."
                 )
             if self.is_disjoint:
                 # Textual Definition of Sibling Shapes:
