@@ -8,24 +8,29 @@ from typing import Dict, List
 
 import rdflib
 
+from rdflib.namespace import XSD
+
 from pyshacl.constraints.constraint_component import ConstraintComponent
-from pyshacl.consts import SH
+from pyshacl.consts import RDF, SH, XSD_WHOLE_INTEGERS
 from pyshacl.errors import ConstraintLoadError, ReportableRuntimeError
 from pyshacl.pytypes import GraphLike
 from pyshacl.rdfutil import stringify_node
 
 
-SH_PatternConstraintComponent = SH.term('PatternConstraintComponent')
-SH_MinLengthConstraintComponent = SH.term('MinLengthConstraintComponent')
-SH_MaxLengthConstraintComponent = SH.term('MaxLengthConstraintComponent')
-SH_LanguageInConstraintComponent = SH.term('LanguageInConstraintComponent')
-SH_UniqueLangConstraintComponent = SH.term('UniqueLangConstraintComponent')
-SH_pattern = SH.term('pattern')
-SH_flags = SH.term('flags')
-SH_minLength = SH.term('minLength')
-SH_maxLength = SH.term('maxLength')
-SH_languageIn = SH.term('languageIn')
-SH_uniqueLang = SH.term('uniqueLang')
+RDF_langString = RDF.langString
+XSD_string = XSD.string
+
+SH_PatternConstraintComponent = SH.PatternConstraintComponent
+SH_MinLengthConstraintComponent = SH.MinLengthConstraintComponent
+SH_MaxLengthConstraintComponent = SH.MaxLengthConstraintComponent
+SH_LanguageInConstraintComponent = SH.LanguageInConstraintComponent
+SH_UniqueLangConstraintComponent = SH.UniqueLangConstraintComponent
+SH_pattern = SH.pattern
+SH_flags = SH.flags
+SH_minLength = SH.minLength
+SH_maxLength = SH.maxLength
+SH_languageIn = SH.languageIn
+SH_uniqueLang = SH.uniqueLang
 
 
 class StringBasedConstraintBase(ConstraintComponent):
@@ -33,6 +38,8 @@ class StringBasedConstraintBase(ConstraintComponent):
     https://www.w3.org/TR/shacl/#core-components-string
     """
 
+    shacl_constraint_component = rdflib.URIRef("urn:notimplemented")
+
     def __init__(self, shape):
         super(StringBasedConstraintBase, self).__init__(shape)
         self.string_rules = []
@@ -46,14 +53,13 @@ class StringBasedConstraintBase(ConstraintComponent):
     def constraint_name(cls):
         raise NotImplementedError()
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        raise NotImplementedError()
-
     @classmethod
     def value_node_to_string(cls, v):
         if isinstance(v, rdflib.Literal):
-            v_string = str(v.value)
+            if v.value is not None and (v.datatype in (None, RDF_langString, XSD_string)):
+                v_string = str(v.value)
+            else:
+                v_string = str(v)
         elif isinstance(v, rdflib.URIRef):
             v_string = str(v)
         else:
@@ -90,6 +96,8 @@ class MinLengthConstraintComponent(StringBasedConstraintBase):
     For each value node v where the length (as defined by the SPARQL STRLEN function) of the string representation of v (as defined by the SPARQL str function) is less than $minLength, or where v is a blank node, there is a validation result with v as sh:value.
     """
 
+    shacl_constraint_component = SH_MinLengthConstraintComponent
+
     def __init__(self, shape):
         super(MinLengthConstraintComponent, self).__init__(shape)
         self.allow_multi_rules = False
@@ -104,6 +112,23 @@ class MinLengthConstraintComponent(StringBasedConstraintBase):
                 "MinLengthConstraintComponent must have at most one sh:minLength predicate.",
                 "https://www.w3.org/TR/shacl/#MinLengthConstraintComponent",
             )
+        for s_r in patterns_found:
+            if (
+                (not isinstance(s_r, rdflib.Literal))
+                or getattr(s_r, "ill_typed", False)
+                or s_r.datatype is None
+                or s_r.datatype not in XSD_WHOLE_INTEGERS
+            ):
+                raise ConstraintLoadError(
+                    "sh:minLength value must be a literal value with an integer.",
+                    "https://www.w3.org/TR/shacl/#MinLengthConstraintComponent",
+                )
+            elif s_r.datatype in (XSD.negativeInteger, XSD.nonPositiveInteger) or s_r.value < 0:
+                raise ConstraintLoadError(
+                    "sh:minLength value must be a positive integer.",
+                    "https://www.w3.org/TR/shacl/#MinLengthConstraintComponent",
+                )
+
         self.string_rules = patterns_found
 
     @classmethod
@@ -114,10 +139,6 @@ class MinLengthConstraintComponent(StringBasedConstraintBase):
     def constraint_name(cls):
         return "MinLengthConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_MinLengthConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         m = "String length not >= {}".format(stringify_node(datagraph, self.string_rules[0]))
         return [rdflib.Literal(m)]
@@ -126,7 +147,7 @@ class MinLengthConstraintComponent(StringBasedConstraintBase):
         reports = []
         non_conformant = False
         assert isinstance(r, rdflib.Literal)
-        min_len = int(r.value)
+        min_len = r.value
         if min_len < 0:
             raise ReportableRuntimeError("Minimum length cannot be less than zero!")
         for f, value_nodes in f_v_dict.items():
@@ -156,6 +177,8 @@ class MaxLengthConstraintComponent(StringBasedConstraintBase):
     For each value node v where the length (as defined by the SPARQL STRLEN function) of the string representation of v (as defined by the SPARQL str function) is greater than $maxLength, or where v is a blank node, there is a validation result with v as sh:value.
     """
 
+    shacl_constraint_component = SH_MaxLengthConstraintComponent
+
     def __init__(self, shape):
         super(MaxLengthConstraintComponent, self).__init__(shape)
         self.allow_multi_rules = False
@@ -170,6 +193,22 @@ class MaxLengthConstraintComponent(StringBasedConstraintBase):
                 "MaxLengthConstraintComponent must have at most one sh:maxLength predicate.",
                 "https://www.w3.org/TR/shacl/#MaxLengthConstraintComponent",
             )
+        for s_r in patterns_found:
+            if (
+                (not isinstance(s_r, rdflib.Literal))
+                or getattr(s_r, "ill_typed", False)
+                or s_r.datatype is None
+                or s_r.datatype not in XSD_WHOLE_INTEGERS
+            ):
+                raise ConstraintLoadError(
+                    "sh:maxLength value must be a literal value with an integer.",
+                    "https://www.w3.org/TR/shacl/#MaxLengthConstraintComponent",
+                )
+            elif s_r.datatype in (XSD.negativeInteger, XSD.nonPositiveInteger) or s_r.value < 0:
+                raise ConstraintLoadError(
+                    "sh:maxLength value must be a positive integer.",
+                    "https://www.w3.org/TR/shacl/#MaxLengthConstraintComponent",
+                )
         self.string_rules = patterns_found
 
     @classmethod
@@ -180,10 +219,6 @@ class MaxLengthConstraintComponent(StringBasedConstraintBase):
     def constraint_name(cls):
         return "MaxLengthConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_MaxLengthConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         m = "String length not <= {}".format(stringify_node(datagraph, self.string_rules[0]))
         return [rdflib.Literal(m)]
@@ -192,7 +227,7 @@ class MaxLengthConstraintComponent(StringBasedConstraintBase):
         reports = []
         non_conformant = False
         assert isinstance(r, rdflib.Literal)
-        max_len = int(r.value)
+        max_len = r.value
         if max_len < 0:
             raise ReportableRuntimeError("Maximum length cannot be less than zero!")
         for f, value_nodes in f_v_dict.items():
@@ -220,6 +255,8 @@ class PatternConstraintComponent(StringBasedConstraintBase):
     For each value node v: A failure MUST be produced if the validation of v as focus node against the property shape $property produces a failure. Otherwise, the validation results are the results of validating v as focus node against the property shape $property.
     """
 
+    shacl_constraint_component = SH_PatternConstraintComponent
+
     def __init__(self, shape):
         super(PatternConstraintComponent, self).__init__(shape)
         patterns_found = list(self.shape.objects(SH_pattern))
@@ -250,10 +287,6 @@ class PatternConstraintComponent(StringBasedConstraintBase):
     def constraint_name(cls):
         return "PatternConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_PatternConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         if len(self.string_rules) < 2:
             m = "Value does not match pattern '{}'".format(str(self.string_rules[0].value))
@@ -304,6 +337,10 @@ class LanguageInConstraintComponent(StringBasedConstraintBase):
     For each value node that is either not a literal or that does not have a language tag matching any of the basic language ranges that are the members of $languageIn following the filtering schema defined by the SPARQL langMatches function, there is a validation result with the value node as sh:value.
     """
 
+    shacl_constraint_component = SH_LanguageInConstraintComponent
+    shape_expecting = False
+    list_taking = True
+
     def __init__(self, shape):
         super(LanguageInConstraintComponent, self).__init__(shape)
         self.allow_multi_rules = False
@@ -328,10 +365,6 @@ class LanguageInConstraintComponent(StringBasedConstraintBase):
     def constraint_name(cls):
         return "LanguageInConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_LanguageInConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         m = "String language is not in {}".format(stringify_node(datagraph, self.string_rules[0]))
         return [rdflib.Literal(m)]
@@ -389,6 +422,8 @@ class UniqueLangConstraintComponent(StringBasedConstraintBase):
     If $uniqueLang is true then for each non-empty language tag that is used by at least two value nodes, there is a validation result.
     """
 
+    shacl_constraint_component = SH_UniqueLangConstraintComponent
+
     def __init__(self, shape):
         super(UniqueLangConstraintComponent, self).__init__(shape)
         self.allow_multi_rules = False
@@ -424,10 +459,6 @@ class UniqueLangConstraintComponent(StringBasedConstraintBase):
     def constraint_name(cls):
         return "UniqueLangConstraintComponent"
 
-    @classmethod
-    def shacl_constraint_class(cls):
-        return SH_UniqueLangConstraintComponent
-
     def make_generic_messages(self, datagraph: GraphLike, focus_node, value_node) -> List[rdflib.Literal]:
         return [rdflib.Literal("More than one String shares the same Language")]
 
