@@ -63,6 +63,7 @@ class Validator(object):
         options_dict.setdefault('use_js', False)
         options_dict.setdefault('iterate_rules', False)
         options_dict.setdefault('abort_on_first', False)
+        options_dict.setdefault('allow_infos', False)
         options_dict.setdefault('allow_warnings', False)
         if 'logger' not in options_dict:
             options_dict['logger'] = logging.getLogger(__name__)
@@ -215,8 +216,16 @@ class Validator(object):
             if inference_option and not self.pre_inferenced and str(inference_option) != "none":
                 if not has_cloned and not self.inplace:
                     the_target_graph = clone_graph(the_target_graph)
+                    has_cloned = True
                 self._run_pre_inference(the_target_graph, inference_option, self.logger)
                 self.pre_inferenced = True
+            if not has_cloned and not self.inplace and self.options['advanced']:
+                # We still need to clone in advanced mode, because of triple rules
+                the_target_graph = clone_graph(the_target_graph)
+                has_cloned = True
+            if not has_cloned and not self.inplace:
+                # No inferencing, no ont_graph, and no advanced mode, now implies inplace mode
+                self.inplace = True
             self._target_graph = the_target_graph
 
         shapes = self.shacl_graph.shapes  # This property getter triggers shapes harvest.
@@ -243,6 +252,7 @@ class Validator(object):
             named_graphs = [the_target_graph]
         reports = []
         abort_on_first: bool = bool(self.options.get("abort_on_first", False))
+        allow_infos: bool = bool(self.options.get("allow_infos", False))
         allow_warnings: bool = bool(self.options.get("allow_warnings", False))
         non_conformant = False
         aborted = False
@@ -252,7 +262,9 @@ class Validator(object):
                 apply_rules(advanced['rules'], g, iterate=iterate_rules)
             try:
                 for s in shapes:
-                    _is_conform, _reports = s.validate(g, abort_on_first=abort_on_first, allow_warnings=allow_warnings)
+                    _is_conform, _reports = s.validate(
+                        g, abort_on_first=abort_on_first, allow_infos=allow_infos, allow_warnings=allow_warnings
+                    )
                     non_conformant = non_conformant or (not _is_conform)
                     reports.extend(_reports)
                     if abort_on_first and non_conformant:
@@ -332,6 +344,7 @@ def validate(
     inference: Optional[str] = None,
     inplace: Optional[bool] = False,
     abort_on_first: Optional[bool] = False,
+    allow_infos: Optional[bool] = False,
     allow_warnings: Optional[bool] = False,
     **kwargs,
 ):
@@ -353,6 +366,8 @@ def validate(
     :type inplace: bool
     :param abort_on_first: Stop evaluating constraints after first violation is found
     :type abort_on_first: bool | None
+    :param allow_infos: Shapes marked with severity of sh:Info will not cause result to be invalid.
+    :type allow_infos: bool | None
     :param allow_warnings: Shapes marked with severity of sh:Warning or sh:Info will not cause result to be invalid.
     :type allow_warnings: bool | None
     :param kwargs:
@@ -409,6 +424,7 @@ def validate(
                 'inference': inference,
                 'inplace': inplace,
                 'abort_on_first': abort_on_first,
+                'allow_infos': allow_infos,
                 'allow_warnings': allow_warnings,
                 'advanced': advanced,
                 'iterate_rules': iterate_rules,
@@ -478,16 +494,16 @@ def compare_validation_reports(report_graph: GraphLike, expected_graph: GraphLik
         )
     expected_conform = next(iter(expected_conforms))
     expected_result_nodes = expected_graph.objects(expected_result, SH_result)
-    expected_result_nodes = set(expected_result_nodes)
-    expected_result_node_count = len(expected_result_nodes)
+    expected_result_nodes_set = set(expected_result_nodes)
+    expected_result_node_count = len(expected_result_nodes_set)
 
     validation_reports = report_graph.subjects(RDF_type, SH_ValidationReport)
-    validation_reports = set(validation_reports)
-    if len(validation_reports) < 1:  # pragma: no cover
+    validation_reports_set = set(validation_reports)
+    if len(validation_reports_set) < 1:  # pragma: no cover
         raise ReportableRuntimeError(
             "Cannot check the validation report, the report graph does not contain a ValidationReport"
         )
-    validation_report = next(iter(validation_reports))
+    validation_report = next(iter(validation_reports_set))
     clean_validation_reports(report_graph, validation_report, expected_graph, expected_result)
     eq = compare_blank_node(report_graph, validation_report, expected_graph, expected_result)
     if eq != 0:
@@ -600,55 +616,55 @@ def check_dash_result(validator: Validator, report_graph: GraphLike, expected_re
         was_default_union = expected_result_graph.default_union
         expected_result_graph.default_union = True  # Force default-union to make all of this a bit easier
     gv_test_cases = expected_result_graph.subjects(RDF_type, DASH_GraphValidationTestCase)
-    gv_test_cases = set(gv_test_cases)
+    gv_test_cases_set = set(gv_test_cases)
     inf_test_cases = expected_result_graph.subjects(RDF_type, DASH_InferencingTestCase)
-    inf_test_cases = set(inf_test_cases)
+    inf_test_cases_set = set(inf_test_cases)
     fn_test_cases = expected_result_graph.subjects(RDF_type, DASH_FunctionTestCase)
-    fn_test_cases = set(fn_test_cases)
-    if len(gv_test_cases) > 0:
-        test_case = next(iter(gv_test_cases))
+    fn_test_cases_set = set(fn_test_cases)
+    if len(gv_test_cases_set) > 0:
+        test_case = next(iter(gv_test_cases_set))
         expected_results = expected_result_graph.objects(test_case, DASH_expectedResult)
-        expected_results = set(expected_results)
-        if len(expected_results) < 1:  # pragma: no cover
+        expected_results_set = set(expected_results)
+        if len(expected_results_set) < 1:  # pragma: no cover
             raise ReportableRuntimeError(
                 "Cannot check the expected result, the given GraphValidationTestCase does not have an expectedResult."
             )
-        expected_result = next(iter(expected_results))
+        expected_result = next(iter(expected_results_set))
         gv_res: Union[bool, None] = compare_validation_reports(report_graph, expected_result_graph, expected_result)
     else:
         gv_res = None
-    if len(inf_test_cases) > 0:
+    if len(inf_test_cases_set) > 0:
         data_graph = validator.target_graph
         if isinstance(data_graph, (rdflib.ConjunctiveGraph, rdflib.Dataset)):
             named_graphs = list(data_graph.contexts())
         else:
             named_graphs = [data_graph]
         inf_res: Union[bool, None] = True
-        for test_case in inf_test_cases:
+        for test_case in inf_test_cases_set:
             expected_results = expected_result_graph.objects(test_case, DASH_expectedResult)
-            expected_results = set(expected_results)
-            if len(expected_results) < 1:  # pragma: no cover
+            expected_results_set = set(expected_results)
+            if len(expected_results_set) < 1:  # pragma: no cover
                 raise ReportableRuntimeError(
                     "Cannot check the expected result, the given InferencingTestCase does not have an expectedResult."
                 )
             found = False
             for g in named_graphs:
-                found = found or compare_inferencing_reports(g, expected_result_graph, expected_results)
+                found = found or compare_inferencing_reports(g, expected_result_graph, expected_results_set)
             inf_res = inf_res and found
     else:
         inf_res = None
-    if len(fn_test_cases) > 0:
+    if len(fn_test_cases_set) > 0:
         data_graph = validator.target_graph
         fns = gather_functions(validator.shacl_graph)
         apply_functions(fns, data_graph)
         fn_res: Union[bool, None] = True
-        for test_case in fn_test_cases:
-            expected_results = set(expected_result_graph.objects(test_case, DASH_expectedResult))
-            if len(expected_results) < 1:  # pragma: no cover
+        for test_case in fn_test_cases_set:
+            expected_results_set = set(expected_result_graph.objects(test_case, DASH_expectedResult))
+            if len(expected_results_set) < 1:  # pragma: no cover
                 raise ReportableRuntimeError(
                     "Cannot check the expected result, the given FunctionTestCase does not have an expectedResult."
                 )
-            expected_result = next(iter(expected_results))
+            expected_result = next(iter(expected_results_set))
             expressions = set(expected_result_graph.objects(test_case, DASH_expression))
             if len(expressions) < 1:
                 raise ReportableRuntimeError(
